class EnemyBullet {
  PVector location, velocity;
  float fuse, speed, rotation, damage, tracking;
  boolean stopped_by_walls;
  PImage img;
  
  EnemyBullet(float x, float y, float _speed, float direction, float range, String img_name, float _damage, float _tracking, boolean _stopped_by_walls) {
    location = new PVector(x, y);
    velocity = new PVector(_speed*cos(radians(direction)), _speed*sin(radians(direction)));
    fuse = range/_speed;
    speed = _speed;
    rotation = direction;
    img = texture_manager.get_texture(img_name);
    damage = _damage;
    tracking = _tracking;
    stopped_by_walls = _stopped_by_walls;
  }
  
  EnemyBullet(float x, float y, float _speed, float range, String img_name, float _damage, float _tracking, boolean _stopped_by_walls) {
    float direction = degrees(atan2(player.y - y, player.x - x));
    location = new PVector(x, y);
    velocity = new PVector(_speed*cos(radians(direction)), _speed*sin(radians(direction)));
    fuse = range/_speed;
    speed = _speed;
    rotation = direction;
    img = texture_manager.get_texture(img_name);
    damage = _damage;
    tracking = _tracking;
    stopped_by_walls = _stopped_by_walls;
  }
  
  void update() {
    track();
    move();
    checkPlayerCollision();
    display();
  }
  
  void track() {
    if (tracking > 0) {
      PVector toTarget = new PVector(player.x - location.x, player.y - location.y);
      toTarget.normalize();
      toTarget.mult(speed);
      velocity.set(lerp(velocity.x, toTarget.x, tracking), lerp(velocity.y, toTarget.y, tracking));
    }
  }
  
  void move() {
    location.add(velocity);
    fuse--;
    if (stopped_by_walls) {
      int dis = 20;
      int offset = 15;
      for (int i = max(0, round(location.x/20)-dis+offset); i < min(round(location.x/20)+dis+offset, blocks.length); i++) {
        for (int j = max(0, round(location.y/20)-dis+offset); j < min(round(location.y/20)+dis+offset, blocks[i].length); j++) {
          if (blocks[i][j] != null && detectCollision(img, location.x, location.y, blocks[i][j].img, blocks[i][j].x, blocks[i][j].y)) {
            fuse = 0;
            return;
          }
        }
      }
    }
  }
  
  void display() {
    pushMatrix();
    translate(location.x, location.y);
    rotate(radians(rotation));
    image(img, 0, 0);
    popMatrix();
  }
  
  void checkPlayerCollision() {
    if (detectCollision(img, location.x, location.y, player.img_left, player.x, player.y)) {
      player.damage(damage);
      fuse = 0;
    }
  }
  
  boolean detectCollision (PImage img1, float x1, float y1, PImage img2, float x2, float y2) {
    if  ((x1 + img1.width/2 >= x2 - img2.width/2) &&
         (x1 - img1.width/2 <= x2 + img2.width/2) &&
         (y1 + img1.height/2 >= y2 - img2.height/2) &&
         (y1 - img1.height/2 <= y2 + img2.height/2))
    {
          return true;
    }
    return false;
  }
  
  boolean should_be_destroyed() {
    if (fuse <= 0) {
      return true;
    }
    return false;
  }
}